@@ -26,7 +26,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.45';
+$VERSION = '1.46';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
@@ -79,7 +79,7 @@ my %faceCategories = (
             this number is unique, and contains the date of manufacture, but doesn't
             necessarily correspond to the camera body number -- this needs to be checked
         },
-        # ie)  "FPX20017035 592D31313034060427796060110384"
+        # eg)  "FPX20017035 592D31313034060427796060110384"
         # "FPX 20495643     592D313335310701318AD010110047" (F40fd)
         #                               yymmdd
         PrintConv => q{
@@ -359,6 +359,26 @@ my %faceCategories = (
     # 0x1150 - Pro Low-light - val=1; Pro Focus - val=2 (ref 7)
     # 0x1151 - Pro Low-light - val=4 (number of pictures taken?); Pro Focus - val=2,3 (ref 7)
     # 0x1152 - Pro Low-light - val=1,3,4 (stacked pictures used?); Pro Focus - val=1,2 (ref 7)
+    0x1201 => { #forum6109
+        Name => 'AdvancedFilter',
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => {
+            0x10000 => 'Pop Color',
+            0x20000 => 'Hi Key',
+            0x30000 => 'Toy Camera',
+            0x40000 => 'Miniature',
+            0x50000 => 'Dynamic Tone',
+            0x60001 => 'Partial Color Red',
+            0x60002 => 'Partial Color Yellow',
+            0x60003 => 'Partial Color Green',
+            0x60004 => 'Partial Color Blue',
+            0x60005 => 'Partial Color Orange',
+            0x60006 => 'Partial Color Purple',
+            0x70000 => 'Soft Focus',
+            0x90000 => 'Low Key',
+        },
+    },
     0x1210 => { #2
         Name => 'ColorMode',
         Writable => 'int16u',
@@ -396,7 +416,7 @@ my %faceCategories = (
     0x1304 => { #PH
         Name => 'GEImageSize',
         Condition => '$$self{Make} =~ /^GENERAL IMAGING/',
-        Format => 'string',
+        Writable => 'string',
         Notes => 'GE models only',
     },
     0x1400 => { #2
@@ -482,9 +502,15 @@ my %faceCategories = (
             2 => 'On (mode 2, shooting only)',
         }],
     },
+    0x1431 => { #forum6109
+        Name => 'Rating',
+        Groups => { 2 => 'Image' },
+        Writable => 'int32u',
+        Priority => 0,
+    },
     0x1436 => { #8
         Name => 'ImageGeneration',
-        Format => 'int16u',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Original Image',
             1 => 'Re-developed from RAW',
@@ -635,6 +661,52 @@ my %faceCategories = (
             return $val;
         },
     },
+    0x2000 => { #9
+        Name => 'WB_GRGBLevelsAuto',
+        Format => 'int16u',
+        Count => 4, # (ignore the duplicate values)
+    },
+    0x2100 => { #9
+        Name => 'WB_GRGBLevelsDaylight',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2200 => { #9
+        Name => 'WB_GRGBLevelsCloudy',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2300 => { #9
+        Name => 'WB_GRGBLevelsDaylightFluor',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2301 => { #9
+        Name => 'WB_GRGBLevelsDayWhiteFluor',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2302 => { #9
+        Name => 'WB_GRGBLevelsWhiteFluorescent',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2310 => { #9
+        Name => 'WB_GRGBLevelsWarmWhiteFluor',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2311 => { #9
+        Name => 'WB_GRGBLevelsLivingRoomWarmWhiteFluor',
+        Format => 'int16u',
+        Count => 4,
+    },
+    0x2400 => { #9
+        Name => 'WB_GRGBLevelsTungsten',
+        Format => 'int16u',
+        Count => 4,
+    },
+    # 0x2f00 => WB_GRGBLevelsCustom: int32u count, then count * (int16u GRGBGRGB), ref 9
     0x2ff0 => {
         Name => 'WB_GRGBLevels',
         Format => 'int16u',
