@@ -8,6 +8,9 @@
 # References:   1) Tae-Sun Park private communication
 #               2) http://www.cybercom.net/~dcoffin/dcraw/
 #               3) Pascal de Bruijn private communication (NX100)
+#               4) Jaroslav Stepanek via rt.cpan.org
+#               5) Niels Kristian Bech Jensen private communication
+#               6) Nick Livchits private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Samsung;
@@ -17,24 +20,34 @@ use vars qw($VERSION %samsungLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.14';
+$VERSION = '1.27';
 
 sub WriteSTMN($$$);
 sub ProcessINFO($$$);
 sub ProcessSamsungIFD($$$);
 
+# Samsung LensType lookup
 %samsungLensTypes = (
+    # (added "Samsung NX" in all of these lens names - ref 4)
     0 => 'Built-in or Manual Lens', #PH (EX1, WB2000)
-    1 => 'Samsung 30mm F2 Pancake',
-    2 => 'Samsung Zoom 18-55mm F3.5-5.6 OIS', # (also version II, ref 1)
-    3 => 'Samsung Zoom 50-200mm F4-5.6 ED OIS',
+    1 => 'Samsung NX 30mm F2 Pancake',
+    2 => 'Samsung NX 18-55mm F3.5-5.6 OIS', # (also version II, ref 1)
+    3 => 'Samsung NX 50-200mm F4-5.6 ED OIS',
     # what about the non-OIS version of the 18-55,
-    # which was supposed to be available before the 20-50?
-    4 => 'Samsung 20-50mm F3.5-5.6 Compact Zoom', #PH
-    5 => 'Samsung 20mm F2.8 Pancake', #PH (guess)
-    # 6 - 'Samsung 85mm F1.4 ED SSA', ? (ref 1 guess)
-    7 => 'Samsung 60mm F2.8 Macro ED OIS SSA', #1
-    8 => 'Samsung 16mm F2.4 Ultra Wide Pancake', #1
+    # which was supposed to be available before the 20-50? - PH
+    4 => 'Samsung NX 20-50mm F3.5-5.6 ED', #PH/4
+    5 => 'Samsung NX 20mm F2.8 Pancake', #PH
+    6 => 'Samsung NX 18-200mm F3.5-6.3 ED OIS', #4
+    7 => 'Samsung NX 60mm F2.8 Macro ED OIS SSA', #1
+    8 => 'Samsung NX 16mm F2.4 Pancake', #1/4
+    9 => 'Samsung NX 85mm F1.4 ED SSA', #4
+    10 => 'Samsung NX 45mm F1.8', #3
+    11 => 'Samsung NX 45mm F1.8 2D/3D', #3
+    12 => 'Samsung NX 12-24mm F4-5.6 ED', #4
+    13 => 'Saumsun NX 16-50mm F2-2.8 S ED OIS', #forum3833
+    14 => 'Samsung NX 10mm F3.5 Fisheye', #5
+    15 => 'Samsung NX 16-50mm F3.5-5.6 Power Zoom ED OIS', #6
+    20 => 'Samsung NX 50-150mm F2.8 S ED OIS', #PH
 );
 
 # range of values for Formats used in encrypted information
@@ -46,7 +59,7 @@ my %formatMinMax = (
 );
 
 # Samsung "STMN" maker notes (ref PH)
-%Image::ExifTool::Samsung::Type1 = (
+%Image::ExifTool::Samsung::Main = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&WriteSTMN,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
@@ -90,7 +103,7 @@ my %formatMinMax = (
 %Image::ExifTool::Samsung::IFD = (
     PROCESS_PROC => \&ProcessSamsungIFD,
     NOTES => q{
-        This is a standard-format IFD found in the Type1 maker notes of some Samsung
+        This is a standard-format IFD found in the maker notes of some Samsung
         models, except that the entry count is a 4-byte integer and the offsets are
         relative to the end of the IFD.  Currently, no tags in this IFD are known,
         so the Unknown (-u) or Verbose (-v) option must be used to see this
@@ -171,7 +184,8 @@ my %formatMinMax = (
         Name => 'LensType',
         Groups => { 2 => 'Camera' },
         Writable => 'int16u',
-        PrintConv => \%samsungLensTypes,
+        Count => -1,
+        PrintConv => [ \%samsungLensTypes ],
     },
     0xa004 => { #1
         Name => 'LensFirmware',
@@ -587,6 +601,103 @@ my %formatMinMax = (
     4 => { Name => 'ThumbnailOffset', IsOffset => 1 },
 );
 
+# Samsung MP4 @sec information (PH - from WB30F sample)
+%Image::ExifTool::Samsung::sec = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    NOTES => q{
+        This information is found in the @sec atom of Samsung MP4 videos from models
+        such as the WB30F.
+    },
+    0x00 => {
+        Name => 'Make',
+        Format => 'string[32]',
+        PrintConv => 'ucfirst(lc($val))',
+    },
+    0x20 => {
+        Name => 'Model',
+        Description => 'Camera Model Name',
+        Format => 'string[32]',
+    },
+    0x200 => { Name => 'ThumbnailWidth',  Format => 'int32u' },
+    0x204 => { Name => 'ThumbnailHeight', Format => 'int32u' },
+    0x208 => { Name => 'ThumbnailLength', Format => 'int32u' }, # (2 bytes too long in my sample)
+    0x20c => {
+        Name => 'ThumbnailImage',
+        Format => 'undef[$val{0x208}]',
+        Notes => 'the THM image, embedded metadata is extracted as the first sub-document',
+        SetBase => 1,
+        RawConv => q{
+            my $pt = $self->ValidateImage(\$val, $tag);
+            if ($pt) {
+                $$self{BASE} += 0x20c;
+                $$self{DOC_NUM} = ++$$self{DOC_COUNT};
+                $self->ExtractInfo($pt, { ReEntry => 1 });
+                $$self{DOC_NUM} = 0;
+            }
+            return $pt;
+        },
+    },
+);
+
+# Samsung MP4 smta information (PH - from SM-C101 sample)
+%Image::ExifTool::Samsung::smta = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Video' },
+    NOTES => q{
+        This information is found in the smta atom of Samsung MP4 videos from models
+        such as the Galaxy S4.
+    },
+    svss => {
+        Name => 'SamsungSvss',
+        SubDirectory => { TagTable => 'Image::ExifTool::Samsung::svss' },
+    },
+    # swtr - 4 bytes, all zero
+    # scid - 8 bytes, all zero
+    # saut - 4 bytes, all zero
+);
+
+# Samsung MP4 svss information (PH - from SM-C101 sample)
+%Image::ExifTool::Samsung::svss = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Video' },
+    NOTES => q{
+        This information is found in the svss atom of Samsung MP4 videos from models
+        such as the Galaxy S4.
+    },
+    # junk - 10240 bytes, all zero
+);
+
+# thumbnail image information found in some MP4 videos
+%Image::ExifTool::Samsung::Thumbnail2 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    FORMAT => 'int32u',
+    1 => 'ThumbnailWidth',
+    2 => 'ThumbnailHeight',
+    3 => 'ThumbnailLength',
+    4 => { Name => 'ThumbnailOffset', IsOffset => 1 },
+);
+
+# information extracted from Samsung trailer (ie. Samsung SM-T805 "Sound & Shot" JPEG)
+%Image::ExifTool::Samsung::Trailer = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Other' },
+    VARS => { NO_ID => 1 },
+    NOTES => q{
+        Tags extracted from the trailer of JPEG images written when using certain
+        features (such as "Sound & Shot" or "Shot & More") from Samsung models such
+        as the Galaxy S4 and Tab S.
+    },
+    # stuff written with "Shot & More" feature
+    '0x0001' => { Name => 'EmbeddedImage', Binary => 1 },
+    '0x0001-name' => 'EmbeddedImageName',
+    # 0x0830 - unknown (164004 bytes, name like "1165724808.pre")
+
+    # stuff written with "Sound & Shot" feature
+    '0x0100' => { Name => 'EmbeddedAudioFile', Binary => 1 },
+    '0x0100-name' => 'EmbeddedAudioFileName',
+    # 0x0800 - SoundShot_Meta_Info (contains only already-extracted sound shot name)
+);
+
 # Samsung composite tags
 %Image::ExifTool::Samsung::Composite = (
     GROUPS => { 2 => 'Image' },
@@ -618,8 +729,8 @@ Image::ExifTool::AddCompositeTags('Image::ExifTool::Samsung');
 #    (in which case the first value of the tag gives the array length)
 sub Crypt($$$@)
 {
-    my ($exifTool, $val, $tagInfo, @salt) = @_;
-    my $key = $$exifTool{EncryptionKey} or return undef;
+    my ($et, $val, $tagInfo, @salt) = @_;
+    my $key = $$et{EncryptionKey} or return undef;
     my $format = $$tagInfo{Writable} || $$tagInfo{Format} or return undef;
     return undef unless $formatMinMax{$format};
     my ($min, $max) = @{$formatMinMax{$format}};
@@ -650,12 +761,12 @@ sub Crypt($$$@)
 # Returns: 1 on success
 sub ProcessINFO($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart};
     my $len = $$dirInfo{DirLen};
     my $end = $pos + $len;
-    $exifTool->VerboseDir('INFO', undef, $len);
+    $et->VerboseDir('INFO', undef, $len);
     while ($pos + 8 <= $end) {
         my $tag = substr($$dataPt, $pos, 4);
         my $val = Get32u($dataPt, $pos + 4);
@@ -664,7 +775,7 @@ sub ProcessINFO($$$)
             $name =~ tr/-_0-9a-zA-Z//dc;
             AddTagToTable($tagTablePtr, $tag, { Name => $name }) if $name;
         }
-        $exifTool->HandleTag($tagTablePtr, $tag, $val);
+        $et->HandleTag($tagTablePtr, $tag, $val);
         $pos += 8;
     }
     return 1;
@@ -675,7 +786,7 @@ sub ProcessINFO($$$)
 # Returns: true on success
 sub ProcessSamsungIFD($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $len = $$dirInfo{DataLen};
     my $pos = $$dirInfo{DirStart};
     return 0 unless $pos + 4 < $len;
@@ -685,10 +796,10 @@ sub ProcessSamsungIFD($$$)
     # (the string "Park Byeongchan" is often found here)
     return 0 unless $buff =~ s/^([^\0])\0\0\0/$1\0$1\0/s;
     my $numEntries = ord $1;
-    if ($$exifTool{HTML_DUMP}) {
+    if ($$et{HTML_DUMP}) {
         my $pt = $$dirInfo{DirStart} + $$dirInfo{DataPos} + $$dirInfo{Base};
-        $exifTool->HDump($pt-44, 44, "MakerNotes header", 'Samsung Type1');
-        $exifTool->HDump($pt, 4, "MakerNotes entries", "Format: int32u\nEntry count: $numEntries");
+        $et->HDump($pt-44, 44, "MakerNotes header", 'Samsung');
+        $et->HDump($pt, 4, "MakerNotes entries", "Format: int32u\nEntry count: $numEntries");
         $$dirInfo{NoDumpEntryCount} = 1;
     }
     substr($$dataPt, $pos, 4) = $buff;      # insert bogus 2-byte entry count
@@ -698,24 +809,178 @@ sub ProcessSamsungIFD($$$)
     $$dirInfo{DataPos} -= $shift;
     $$dirInfo{DirStart} += 2;       # start at bogus entry count
     $$dirInfo{ZeroOffsetOK} = 1;    # disable check for zero offset
-    delete $$exifTool{NO_UNKNOWN};  # (set for BinaryData, but not for EXIF IFD's)
-    my $rtn = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+    delete $$et{NO_UNKNOWN};  # (set for BinaryData, but not for EXIF IFD's)
+    my $rtn = Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
     substr($$dataPt, $pos + 2, 1) = "\0";   # remove bogus count
     return $rtn;
 }
 
+#------------------------------------------------------------------------------
+# Read/write Samsung trailer (ie. "Sound & Shot" written by Galaxy Tab S (SM-T805))
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
+# Returns: 1 on success, 0 not valid Samsung trailer, or -1 error writing
+# - updates DataPos to point to start of Samsung trailer
+# - updates DirLen to existing trailer length
+sub ProcessSamsung($$$)
+{
+    my ($et, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $offset = $$dirInfo{Offset} || 0;
+    my $outfile = $$dirInfo{OutFile};
+    my $verbose = $et->Options('Verbose');
+    my $unknown = $et->Options('Unknown');
+    my ($buff, $buf2, $index, $offsetPos, $audioNOff, $audioSize);
+
+    return 0 unless $raf->Seek(-6-$offset, 2) and $raf->Read($buff, 6) == 6 and
+                    ($buff eq 'QDIOBS' or $buff eq "\0\0SEFT");
+    my $endPos = $raf->Tell();
+    $raf->Seek(-2, 1) or return 0 if $buff eq 'QDIOBS'; # rewind to before 'BS'
+    my $blockEnd = $raf->Tell();
+    SetByteOrder('II');
+
+    # read blocks backward until we find the SEFH/SEFT block
+    # (the only other block I have seen is QDIO/QDIO)
+SamBlock:
+    for (;;) {
+        last unless $raf->Seek($blockEnd-8, 0) and $raf->Read($buff, 8) == 8;
+        my $type = substr($buff, 4);
+        last unless $type =~ /^\w+$/;
+        my $len = Get32u(\$buff, 0);
+        last unless $len < 0x10000 and $len >= 4 and $len + 8 < $blockEnd;
+        last unless $raf->Seek(-8-$len, 1) and $raf->Read($buff, $len) == $len;
+        $blockEnd -= $len + 8;
+        unless ($type eq 'SEFT') {  # look for directory block (ends with "SEFT")
+            next unless $outfile and $type eq 'QDIO';
+            # QDIO block format:
+            #   0 - 'QDIO'
+            #   4 - int32u: 101 (version)
+            #   8 - int32u: 1
+            #  12 - int32u: absolute offset of audio file start (augh!!)
+            #  16 - int32u: absolute offset of audio file end (augh!!)
+            #  20 - int32u: 20 (QDIO block length minus 8)
+            #  24 - 'QDIO'
+            if ($len == 20) {
+                # save position of audio file offset in QDIO block
+                $offsetPos = $endPos - $raf->Tell() + $len - 12;
+            } else {
+                $et->Error('Unsupported Samsung trailer QDIO block', 1);
+            }
+            next;
+        }
+        last unless $buff =~ /^SEFH/ and $len >= 12;   # validate SEFH header
+        my $dirPos = $raf->Tell() - $len;
+        # my $ver = Get32u(\$buff, 0x04);  # version (=101)
+        my $count = Get32u(\$buff, 0x08);
+        last if 12 + 12 * $count > $len;
+        my $tagTablePtr = GetTagTable('Image::ExifTool::Samsung::Trailer');
+
+        # scan ahead quickly to look for the block where the data comes first
+        # (have only seen this to be the first in the directory, but just in case)
+        my $firstBlock = 0;
+        for ($index=0; $index<$count; ++$index) {
+            my $entry = 12 + 12 * $index;
+            my $noff = Get32u(\$buff, $entry + 4);  # negative offset
+            $firstBlock = $noff if $firstBlock < $noff;
+        }
+        # save trailer position and length
+        my $dataPos = $$dirInfo{DataPos} = $dirPos - $firstBlock;
+        my $dirLen = $$dirInfo{DirLen} = $endPos - $dataPos;
+        if (($verbose or $$et{HTML_DUMP}) and not $outfile) {
+            $et->DumpTrailer($dirInfo);
+            return 1 if $$et{HTML_DUMP};
+        }
+        # read through the SEFH/SEFT directory entries
+        for ($index=0; $index<$count; ++$index) {
+            my $entry = 12 + 12 * $index;
+            # first 2 bytes always 0 (may be part of block type)
+            my $type = Get16u(\$buff, $entry + 2);  # block type
+            my $noff = Get32u(\$buff, $entry + 4);  # negative offset
+            my $size = Get32u(\$buff, $entry + 8);  # block size
+            last SamBlock if $noff > $dirPos or $size > $noff or $size < 8;
+            $firstBlock = $noff if $firstBlock < $noff;
+            if ($outfile) {
+                next unless $type == 0x0100 and not $audioNOff;
+                # save offset and length of first audio file for QDIO block
+                last unless $raf->Seek($dirPos-$noff, 0) and $raf->Read($buf2, 8) == 8;
+                $len = Get32u(\$buf2, 4);
+                $audioNOff = $noff - 8 - $len;   # negative offset to start of audio data
+                $audioSize = $size - 8 - $len;
+                next;
+            }
+            # add unknown tags if necessary
+            my $tag = sprintf("0x%.4x", $type);
+            unless ($$tagTablePtr{$tag}) {
+                next unless $unknown or $verbose;
+                my %tagInfo = (
+                    Name        => "SamsungTrailer_$tag",
+                    Description => "Samsung Trailer $tag",
+                    Unknown     => 1,
+                    Binary      => 1,
+                );
+                AddTagToTable($tagTablePtr, $tag, \%tagInfo);
+                my %tagInfo2 = (
+                    Name        => "SamsungTrailer_${tag}Name",
+                    Description => "Samsung Trailer $tag Name",
+                    Unknown     => 1,
+                );
+                AddTagToTable($tagTablePtr, "$tag-name", \%tagInfo2);
+            }
+            last unless $raf->Seek($dirPos-$noff, 0) and $raf->Read($buf2, $size) == $size;
+            # (could validate the first 4 bytes of the block because they
+            # are the same as the first 4 bytes of the directory entry)
+            $len = Get32u(\$buf2, 4);
+            last if $len + 8 > $size;
+            # extract tag name and value
+            $et->HandleTag($tagTablePtr, "$tag-name", undef,
+                DataPt  => \$buf2,
+                DataPos => $dirPos - $noff,
+                Start   => 8,
+                Size    => $len,
+            );
+            $et->HandleTag($tagTablePtr, $tag, undef,
+                DataPt  => \$buf2,
+                DataPos => $dirPos - $noff,
+                Start   => 8 + $len,
+                Size    => $size - (8 + $len),
+            );
+        }
+        if ($outfile) {
+            last unless $raf->Seek($dataPos, 0) and $raf->Read($buff, $dirLen) == $dirLen;
+            # adjust the absolute offset in the QDIO block if necessary
+            if ($offsetPos and $audioNOff) {
+                # initialize the audio file start/end position in the QDIO block
+                my $newPos = Tell($outfile) + $dirPos - $audioNOff - $dataPos;
+                Set32u($newPos, \$buff, length($buff) - $offsetPos);
+                Set32u($newPos + $audioSize, \$buff, length($buff) - $offsetPos + 4);
+                # add a fixup so the calling routine can apply further shifts if necessary
+                require Image::ExifTool::Fixup;
+                my $fixup = $$dirInfo{Fixup};
+                $fixup or $fixup = $$dirInfo{Fixup} = new Image::ExifTool::Fixup;
+                $fixup->AddFixup(length($buff) - $offsetPos);
+                $fixup->AddFixup(length($buff) - $offsetPos + 4);
+            }
+            $et->VPrint(0, "Writing Samsung trailer ($dirLen bytes)\n") if $verbose;
+            Write($$dirInfo{OutFile}, $buff) or return -1;
+            return 1;
+        }
+        return 1;
+    }
+    $et->Warn('Error processing Samsung trailer',1);
+    return 0;
+}
+
 #------------------------------------------------------------------------------
 # Write Samsung STMN maker notes
 # Inputs: 0) ExifTool object ref, 1) source dirInfo ref, 2) tag table ref
 # Returns: Binary data block or undefined on error
 sub WriteSTMN($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     # create a Fixup for the PreviewImage
     $$dirInfo{Fixup} = new Image::ExifTool::Fixup;
-    my $val = Image::ExifTool::WriteBinaryData($exifTool, $dirInfo, $tagTablePtr);
+    my $val = Image::ExifTool::WriteBinaryData($et, $dirInfo, $tagTablePtr);
     # force PreviewImage into the trailer even if it fits in EXIF segment
-    $$exifTool{PREVIEW_INFO}{IsTrailer} = 1 if $$exifTool{PREVIEW_INFO};
+    $$et{PREVIEW_INFO}{IsTrailer} = 1 if $$et{PREVIEW_INFO};
     return $val;
 }
 
@@ -738,7 +1003,7 @@ Samsung maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
